<?php

//  agencies.getStats
$agenciesGetStats_sig = array(array("struct", "array", 'string', 'string'));
$agenciesGetStats_doc = "Accepts 3 parameters, 1st: an array of agency IDs, " .
                        "2nd: start data string, 3rd: end date string, and returns a struct of agency stats.";
$agenciesGetStats_alias = 'agencies.getStats';

/**
 * Returns stats for a range of agencies.
 *
 * @param object $msg   An rpc message object
 * @return a hash of stats keyed by date
            - date
                - hour
                    - agency id
                        - totalImpressions
                        - totalClicks
 *
 *
        Array
        (
            [2005-05-22] => Array
                (
                    [10] => Array
                        (
                            [1] => Array
                                (
                                    [totalImpressions] => 200
                                    [totalClicks] => 200
                                )
        
                            [2] => Array
                                (
                                    [totalImpressions] => 100
                                    [totalClicks] => 100
                                )
                        )
        
                    [11] => Array
                    ...

 */
function Max_RPC_Server_agenciesGetStats($msg)
{   
    global $XML_RPC_erruser;

    $aAgencyIds = $msg->getParam(0);
    $startDate = $msg->getParam(1);
    $endDate = $msg->getParam(2);
    
    $count = $aAgencyIds->arraysize();
    for ($x = 0; $x < $count; $x++) {
        $rpcValue = $aAgencyIds->arraymem($x);
        $retIds[] = $rpcValue->scalarVal();
    }
    
    //  check for valid input
    if (is_array($retIds) && count($retIds) && elementsAreInts($retIds)) {
    
        require_once MAX_PATH . '/lib/max/Admin_DA.php';
        $ret = Admin_DA::getAgenciesCampaignStats($retIds, $startDate->scalarVal(), $endDate->scalarVal());
       
        //  check for valid DB results
        if (PEAR::isError($ret)) {
            $errorCode = $XML_RPC_erruser + $ret->getCode();
            $errorMsg  = 'PEAR error: ' . $ret->getMessage();
            return new XML_RPC_Response(0, $errorCode, $errorMsg);
        }
    
        //  wrap data structure as XML_RPC values
        foreach ($ret as $date => $aHours) {
            foreach ($aHours as $hour => $aAgencies) {
                foreach ($aAgencies as $agencyId => $aStats) {
                    $aStats['totalImpressions'] = new XML_RPC_Value($aStats['totalImpressions'], "int");
                    $aStats['totalClicks'] = new XML_RPC_Value($aStats['totalClicks'], "int");
                    $ret[$date][$hour][$agencyId] = new XML_RPC_Value($aStats, "struct");
                }
                $ret[$date][$hour] = new XML_RPC_Value($ret[$date][$hour], "struct");
            }
            $ret[$date] = new XML_RPC_Value($ret[$date], "struct");
        }
        $result = new XML_RPC_Value($ret, "struct");
        $return = new XML_RPC_Response($result);
    } else {
        $errorCode = $XML_RPC_erruser + 3;
        $errorMsg  = 'Incorrect first parameter, expected an array of agency ID ints';
        $return = new XML_RPC_Response(0, $errorCode, $errorMsg);
    }
    return $return;
}

function elementsAreInts($array)
{
    foreach ($array as $element) {
        if (!(is_int($element))) {
            return false;   
        }
    }
    return true;
}

?>